@@ -1,14 +1,18 @@
 import json
-import boto3
 import logging
+from abc import ABC, abstractmethod
+from typing import Any, Dict, Optional, cast
 
+import boto3
 from botocore.config import Config
+
 from samtranslator.feature_toggle.dialup import (
     DisabledDialup,
-    ToggleDialup,
     SimpleAccountPercentileDialup,
+    ToggleDialup,
 )
 from samtranslator.metrics.method_decorator import cw_timer
+from samtranslator.utils.constants import BOTO3_CONNECT_TIMEOUT
 
 LOG = logging.getLogger(__name__)
 
@@ -24,7 +28,13 @@ class FeatureToggle:
         "account-percentile": SimpleAccountPercentileDialup,
     }
 
-    def __init__(self, config_provider, stage, account_id, region):  # type: ignore[no-untyped-def]
+    def __init__(
+        self,
+        config_provider: "FeatureToggleConfigProvider",
+        stage: Optional[str],
+        account_id: Optional[str],
+        region: Optional[str],
+    ) -> None:
         self.feature_config = config_provider.config
         self.stage = stage
         self.account_id = account_id
@@ -48,7 +58,7 @@ class FeatureToggle:
         LOG.warning("Dialup type '{}' is None or is not supported.".format(dialup_type))
         return DisabledDialup(region_config)  # type: ignore[no-untyped-call]
 
-    def is_enabled(self, feature_name):  # type: ignore[no-untyped-def]
+    def is_enabled(self, feature_name: str) -> bool:
         """
         To check if feature is available
 
@@ -80,21 +90,19 @@ class FeatureToggle:
 
         dialup = self._get_dialup(region_config, feature_name=feature_name)  # type: ignore[no-untyped-call]
         LOG.info("Using Dialip {}".format(dialup))
-        is_enabled = dialup.is_enabled()
+        is_enabled: bool = dialup.is_enabled()
 
         LOG.info("Feature '{}' is enabled: '{}'".format(feature_name, is_enabled))
         return is_enabled
 
 
-class FeatureToggleConfigProvider:
+class FeatureToggleConfigProvider(ABC):
     """Interface for all FeatureToggle config providers"""
 
-    def __init__(self) -> None:
-        pass
-
     @property
-    def config(self):  # type: ignore[no-untyped-def]
-        raise NotImplementedError
+    @abstractmethod
+    def config(self) -> Dict[str, Any]:
+        pass
 
 
 class FeatureToggleDefaultConfigProvider(FeatureToggleConfigProvider):
@@ -104,7 +112,7 @@ class FeatureToggleDefaultConfigProvider(FeatureToggleConfigProvider):
         FeatureToggleConfigProvider.__init__(self)
 
     @property
-    def config(self):  # type: ignore[no-untyped-def]
+    def config(self) -> Dict[str, Any]:
         return {}
 
 
@@ -115,17 +123,17 @@ class FeatureToggleLocalConfigProvider(FeatureToggleConfigProvider):
         FeatureToggleConfigProvider.__init__(self)
         with open(local_config_path, "r", encoding="utf-8") as f:
             config_json = f.read()
-        self.feature_toggle_config = json.loads(config_json)
+        self.feature_toggle_config = cast(Dict[str, Any], json.loads(config_json))
 
     @property
-    def config(self):  # type: ignore[no-untyped-def]
+    def config(self) -> Dict[str, Any]:
         return self.feature_toggle_config
 
 
 class FeatureToggleAppConfigConfigProvider(FeatureToggleConfigProvider):
     """Feature toggle config provider which loads config from AppConfig."""
 
-    @cw_timer(prefix="External", name="AppConfig")  # type: ignore[misc]
+    @cw_timer(prefix="External", name="AppConfig")
     def __init__(self, application_id, environment_id, configuration_profile_id, app_config_client=None):  # type: ignore[no-untyped-def]
         FeatureToggleConfigProvider.__init__(self)
         try:
@@ -133,9 +141,11 @@ class FeatureToggleAppConfigConfigProvider(FeatureToggleConfigProvider):
             # Lambda function has 120 seconds limit
             # (5 + 5) * 2, 20 seconds maximum timeout duration
             # In case of high latency from AppConfig, we can always fall back to use an empty config and continue transform
-            client_config = Config(connect_timeout=5, read_timeout=5, retries={"total_max_attempts": 2})
+            client_config = Config(
+                connect_timeout=BOTO3_CONNECT_TIMEOUT, read_timeout=5, retries={"total_max_attempts": 2}
+            )
             self.app_config_client = (
-                boto3.client("appconfig", config=client_config) if not app_config_client else app_config_client
+                app_config_client if app_config_client else boto3.client("appconfig", config=client_config)
             )
             response = self.app_config_client.get_configuration(
                 Application=application_id,
@@ -144,13 +154,13 @@ class FeatureToggleAppConfigConfigProvider(FeatureToggleConfigProvider):
                 ClientId="FeatureToggleAppConfigConfigProvider",
             )
             binary_config_string = response["Content"].read()
-            self.feature_toggle_config = json.loads(binary_config_string.decode("utf-8"))
+            self.feature_toggle_config = cast(Dict[str, Any], json.loads(binary_config_string.decode("utf-8")))
             LOG.info("Finished loading feature toggle config from AppConfig.")
         except Exception as ex:
             LOG.error("Failed to load config from AppConfig: {}. Using empty config.".format(ex))
             # There is chance that AppConfig is not available in a particular region.
-            self.feature_toggle_config = json.loads("{}")
+            self.feature_toggle_config = {}
 
     @property
-    def config(self):  # type: ignore[no-untyped-def]
+    def config(self) -> Dict[str, Any]:
         return self.feature_toggle_config
