@@ -1,16 +1,26 @@
 """ CloudFormation Resource serialization, deserialization, and validation """
-import re
 import inspect
-from typing import Any, Callable, Dict, List, Optional, Union
+import re
+from abc import ABC, ABCMeta, abstractmethod
+from contextlib import suppress
+from typing import Any, Callable, Dict, List, Optional, Tuple, Type, TypeVar
 
-from samtranslator.intrinsics.resolver import IntrinsicsResolver
-from samtranslator.model.exceptions import ExpectedType, InvalidResourceException, InvalidResourcePropertyTypeException
+from pydantic import BaseModel
+from pydantic.error_wrappers import ValidationError
+
+from samtranslator.model.exceptions import (
+    ExpectedType,
+    InvalidResourceException,
+    InvalidResourcePropertyTypeException,
+)
+from samtranslator.model.tags.resource_tagging import get_tag_list
 from samtranslator.model.types import IS_DICT, IS_STR, Validator, any_type, is_type
 from samtranslator.plugins import LifeCycleEvents
-from samtranslator.model.tags.resource_tagging import get_tag_list
+
+RT = TypeVar("RT", bound=BaseModel)  # return type
 
 
-class PropertyType(object):
+class PropertyType:
     """Stores validation information for a CloudFormation resource property.
 
     The attribute "expected_type" is only used by InvalidResourcePropertyTypeException
@@ -64,7 +74,18 @@ class PassThroughProperty(PropertyType):
         super().__init__(required, any_type(), False)
 
 
-class Resource(object):
+class GeneratedProperty(PropertyType):
+    """
+    Property of a generated CloudFormation resource.
+    """
+
+    def __init__(self) -> None:
+        # Intentionally the most lenient; we don't want the risk of potential
+        # runtime exceptions, and the object attributes are statically typed
+        super().__init__(False, any_type(), False)
+
+
+class Resource(ABC):
     """A Resource object represents an abstract entity that contains a Type and a Properties object. They map well to
     CloudFormation resources as well sub-types like AWS::Lambda::Function or `Events` section of
     AWS::Serverless::Function.
@@ -103,7 +124,7 @@ class Resource(object):
 
     def __init__(
         self,
-        logical_id: str,
+        logical_id: Optional[Any],
         relative_id: Optional[str] = None,
         depends_on: Optional[List[str]] = None,
         attributes: Optional[Dict[str, Any]] = None,
@@ -116,8 +137,7 @@ class Resource(object):
         :param depends_on Value of DependsOn resource attribute
         :param attributes Dictionary of resource attributes and their values
         """
-        self._validate_logical_id(logical_id)  # type: ignore[no-untyped-call]
-        self.logical_id = logical_id
+        self.logical_id = self._validate_logical_id(logical_id)
         self.relative_id = relative_id
         self.depends_on = depends_on
 
@@ -138,7 +158,7 @@ class Resource(object):
         return tuple(cls._supported_resource_attributes)
 
     @classmethod
-    def get_pass_through_attributes(cls):  # type: ignore[no-untyped-def]
+    def get_pass_through_attributes(cls) -> Tuple[str, ...]:
         """
         A getter method for the resource attributes to be passed to auto-generated resources
         returns: a tuple that contains the name of all pass through attributes
@@ -146,7 +166,7 @@ class Resource(object):
         return tuple(cls._pass_through_attributes)
 
     @classmethod
-    def from_dict(cls, logical_id, resource_dict, relative_id=None, sam_plugins=None):  # type: ignore[no-untyped-def]
+    def from_dict(cls, logical_id: str, resource_dict: Dict[str, Any], relative_id: Optional[str] = None, sam_plugins=None) -> "Resource":  # type: ignore[no-untyped-def]
         """Constructs a Resource object with the given logical id, based on the given resource dict. The resource dict
         is the value associated with the logical id in a CloudFormation template's Resources section, and takes the
         following format. ::
@@ -196,8 +216,8 @@ class Resource(object):
         resource.validate_properties()
         return resource
 
-    @classmethod
-    def _validate_logical_id(cls, logical_id):  # type: ignore[no-untyped-def]
+    @staticmethod
+    def _validate_logical_id(logical_id: Optional[Any]) -> str:
         """Validates that the provided logical id is an alphanumeric string.
 
         :param str logical_id: the logical id to validate
@@ -206,9 +226,12 @@ class Resource(object):
         :raises TypeError: if the logical id is invalid
         """
         pattern = re.compile(r"^[A-Za-z0-9]+$")
-        if logical_id is not None and pattern.match(logical_id):
-            return True
-        raise InvalidResourceException(logical_id, "Logical ids must be alphanumeric.")
+        if isinstance(logical_id, str) and pattern.match(logical_id):
+            return logical_id
+        # TODO: Doing validation in this class is kind of off,
+        # we need to surface this validation to where the template is loaded
+        # or the logical IDs are generated.
+        raise InvalidResourceException(str(logical_id), "Logical ids must be alphanumeric.")
 
     @classmethod
     def _validate_resource_dict(cls, logical_id, resource_dict):  # type: ignore[no-untyped-def]
@@ -254,11 +277,11 @@ class Resource(object):
         """
         self.validate_properties()
 
-        resource_dict = self._generate_resource_dict()  # type: ignore[no-untyped-call]
+        resource_dict = self._generate_resource_dict()
 
         return {self.logical_id: resource_dict}
 
-    def _generate_resource_dict(self):  # type: ignore[no-untyped-def]
+    def _generate_resource_dict(self) -> Dict[str, Any]:
         """Generates the resource dict for this Resource, the value associated with the logical id in a CloudFormation
         template's Resources section.
 
@@ -273,7 +296,7 @@ class Resource(object):
         resource_dict.update(self.resource_attributes)
 
         properties_dict = {}
-        for name in self.property_types.keys():
+        for name in self.property_types:
             value = getattr(self, name)
             if value is not None:
                 properties_dict[name] = value
@@ -290,8 +313,8 @@ class Resource(object):
         :param value: the value of the attribute to be set
         :raises InvalidResourceException: if an invalid property is provided
         """
-        if name in self._keywords or name in self.property_types.keys():
-            return super(Resource, self).__setattr__(name, value)
+        if name in self._keywords or name in self.property_types:
+            return super().__setattr__(name, value)
 
         raise InvalidResourceException(
             self.logical_id,
@@ -300,6 +323,25 @@ class Resource(object):
             ),
         )
 
+    # Note: For compabitliy issue, we should ONLY use this with new abstraction/resources.
+    def validate_properties_and_return_model(self, cls: Type[RT]) -> RT:
+        """
+        Given a resource properties, return a typed object from the definitions of SAM schema model
+
+        param:
+            resource_properties: properties from input template
+            cls: schema models
+        """
+        try:
+            return cls.parse_obj(self._generate_resource_dict()["Properties"])
+        except ValidationError as e:
+            error_properties: str = ""
+            with suppress(KeyError):
+                error_properties = ", ".join([str(error["loc"][0]) for error in e.errors()])
+            raise InvalidResourceException(
+                self.logical_id, f"Given resource property '{error_properties}' is invalid"
+            ) from e
+
     def validate_properties(self) -> None:
         """Validates that the required properties for this Resource have been populated, and that all properties have
         valid values.
@@ -336,7 +378,7 @@ class Resource(object):
         """
 
         if attr not in self._supported_resource_attributes:
-            raise KeyError("Unsupported resource attribute specified: %s" % attr)
+            raise KeyError(f"Unsupported resource attribute specified: {attr}")
 
         self.resource_attributes[attr] = value
 
@@ -347,7 +389,7 @@ class Resource(object):
         :return: Value of the attribute, if set in the resource. None otherwise
         """
         if attr not in self.resource_attributes:
-            raise KeyError("%s is not in resource attributes" % attr)
+            raise KeyError(f"{attr} is not in resource attributes")
 
         return self.resource_attributes[attr]
 
@@ -370,10 +412,10 @@ class Resource(object):
 
         :return: Dictionary that will resolve to value of the attribute when CloudFormation stack update is executed
         """
+        if attr_name not in self.runtime_attrs:
+            raise KeyError(f"{attr_name} attribute is not supported for resource {self.resource_type}")
 
-        if attr_name in self.runtime_attrs:
-            return self.runtime_attrs[attr_name](self)
-        raise NotImplementedError(f"{attr_name} attribute is not implemented for resource {self.resource_type}")
+        return self.runtime_attrs[attr_name](self)
 
     def get_passthrough_resource_attributes(self) -> Dict[str, Any]:
         """
@@ -383,13 +425,13 @@ class Resource(object):
         :return: Dictionary of resource attributes.
         """
         attributes = {}
-        for resource_attribute in self.get_pass_through_attributes():  # type: ignore[no-untyped-call]
+        for resource_attribute in self.get_pass_through_attributes():
             if resource_attribute in self.resource_attributes:
                 attributes[resource_attribute] = self.resource_attributes.get(resource_attribute)
         return attributes
 
 
-class ResourceMacro(Resource):
+class ResourceMacro(Resource, metaclass=ABCMeta):
     """A ResourceMacro object represents a CloudFormation macro. A macro appears in the CloudFormation template in the
     "Resources" mapping, but must be expanded into one or more vanilla CloudFormation resources before a stack can be
     created from it.
@@ -408,6 +450,7 @@ class ResourceMacro(Resource):
         """
         return {}
 
+    @abstractmethod
     def to_cloudformation(self, **kwargs: Any) -> List[Any]:
         """Returns a list of Resource instances, representing vanilla CloudFormation resources, to which this macro
         expands. The caller should be able to update their template with the expanded resources by calling
@@ -416,10 +459,9 @@ class ResourceMacro(Resource):
         :param dict kwargs
         :returns: a list of vanilla CloudFormation Resource instances, to which this macro expands
         """
-        raise NotImplementedError("Method to_cloudformation() must be implemented in a subclass of ResourceMacro")
 
 
-class SamResourceMacro(ResourceMacro):
+class SamResourceMacro(ResourceMacro, metaclass=ABCMeta):
     """ResourceMacro that specifically refers to SAM (AWS::Serverless::*) resources."""
 
     # SAM resources can provide a list of properties that they expose. These properties usually resolve to
@@ -465,9 +507,9 @@ class SamResourceMacro(ResourceMacro):
         # Create a map of {ResourceType: LogicalId} for quick access
         resource_id_by_type = {resource.resource_type: resource.logical_id for resource in generated_cfn_resources}
 
-        for property, cfn_type in self.referable_properties.items():
+        for property_name, cfn_type in self.referable_properties.items():
             if cfn_type in resource_id_by_type:
-                supported_resource_refs.add(self.logical_id, property, resource_id_by_type[cfn_type])
+                supported_resource_refs.add(self.logical_id, property_name, resource_id_by_type[cfn_type])
 
         return supported_resource_refs
 
@@ -501,25 +543,8 @@ class SamResourceMacro(ResourceMacro):
                 "input.",
             )
 
-    def _resolve_string_parameter(
-        self,
-        intrinsics_resolver: IntrinsicsResolver,
-        parameter_value: Optional[Union[str, Dict[str, Any]]],
-        parameter_name: str,
-    ) -> Optional[Union[str, Dict[str, Any]]]:
-        if not parameter_value:
-            return parameter_value
-        value = intrinsics_resolver.resolve_parameter_refs(parameter_value)
 
-        if not isinstance(value, str) and not isinstance(value, dict):
-            raise InvalidResourceException(
-                self.logical_id,
-                "Could not resolve parameter for '{}' or parameter is not a String.".format(parameter_name),
-            )
-        return value
-
-
-class ResourceTypeResolver(object):
+class ResourceTypeResolver:
     """ResourceTypeResolver maps Resource Types to Resource classes, e.g. AWS::Serverless::Function to
     samtranslator.model.sam_resources.SamFunction."""
 
@@ -534,7 +559,7 @@ class ResourceTypeResolver(object):
             for _, resource_class in inspect.getmembers(
                 module,
                 lambda cls: inspect.isclass(cls)
-                and cls.__module__ == module.__name__
+                and cls.__module__ == module.__name__  # noqa: function-uses-loop-variable
                 and hasattr(cls, "resource_type"),
             ):
                 self.resource_types[resource_class.resource_type] = resource_class
@@ -579,16 +604,33 @@ class ResourceResolver:
         """Return a dictionary of all resources from the SAM template."""
         return self.resources
 
-    def get_resource_by_logical_id(self, input: str) -> Dict[str, Any]:
+    def get_resource_by_logical_id(self, _input: str) -> Dict[str, Any]:
         """
         Recursively find resource with matching Logical ID that are present in the template and returns the value.
         If it is not in template, this method simply returns the input unchanged.
 
-        :param input: Logical ID of a resource
+        :param _input: Logical ID of a resource
 
         :param resources: Dictionary of the resource from the SAM template
         """
-        if not isinstance(input, str):
-            raise TypeError("Invalid logical ID '{}'. Expected a string.".format(input))
+        if not isinstance(_input, str):
+            raise TypeError("Invalid logical ID '{}'. Expected a string.".format(_input))
 
-        return self.resources.get(input, None)
+        return self.resources.get(_input, None)
+
+
+__all__: List[str] = [
+    "IS_DICT",
+    "IS_STR",
+    "Validator",
+    "any_type",
+    "is_type",
+    "PropertyType",
+    "Property",
+    "PassThroughProperty",
+    "Resource",
+    "ResourceMacro",
+    "SamResourceMacro",
+    "ResourceTypeResolver",
+    "ResourceResolver",
+]
