@@ -1,12 +1,11 @@
 import json
 from re import match
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Union
 
-from samtranslator.model import PropertyType, Resource
+from samtranslator.model import GeneratedProperty, Resource
 from samtranslator.model.exceptions import InvalidResourceException
 from samtranslator.model.intrinsics import fnSub, ref
-from samtranslator.model.types import IS_DICT, IS_STR, is_type, list_of, one_of
-from samtranslator.schema.common import PassThrough
+from samtranslator.model.types import PassThrough
 from samtranslator.translator import logical_id_generator
 from samtranslator.translator.arn_generator import ArnGenerator
 from samtranslator.utils.py27hash_fix import Py27Dict, Py27UniStr
@@ -16,18 +15,18 @@ from samtranslator.validator.value_validator import sam_expect
 class ApiGatewayRestApi(Resource):
     resource_type = "AWS::ApiGateway::RestApi"
     property_types = {
-        "Body": PropertyType(False, IS_DICT),
-        "BodyS3Location": PropertyType(False, IS_DICT),
-        "CloneFrom": PropertyType(False, IS_STR),
-        "Description": PropertyType(False, IS_STR),
-        "FailOnWarnings": PropertyType(False, is_type(bool)),
-        "Name": PropertyType(False, IS_STR),
-        "Parameters": PropertyType(False, IS_DICT),
-        "EndpointConfiguration": PropertyType(False, IS_DICT),
-        "BinaryMediaTypes": PropertyType(False, is_type(list)),
-        "MinimumCompressionSize": PropertyType(False, is_type(int)),
-        "Mode": PropertyType(False, IS_STR),
-        "ApiKeySourceType": PropertyType(False, IS_STR),
+        "Body": GeneratedProperty(),
+        "BodyS3Location": GeneratedProperty(),
+        "CloneFrom": GeneratedProperty(),
+        "Description": GeneratedProperty(),
+        "FailOnWarnings": GeneratedProperty(),
+        "Name": GeneratedProperty(),
+        "Parameters": GeneratedProperty(),
+        "EndpointConfiguration": GeneratedProperty(),
+        "BinaryMediaTypes": GeneratedProperty(),
+        "MinimumCompressionSize": GeneratedProperty(),
+        "Mode": GeneratedProperty(),
+        "ApiKeySourceType": GeneratedProperty(),
     }
 
     Body: Optional[Dict[str, Any]]
@@ -49,30 +48,32 @@ class ApiGatewayRestApi(Resource):
 class ApiGatewayStage(Resource):
     resource_type = "AWS::ApiGateway::Stage"
     property_types = {
-        "AccessLogSetting": PropertyType(False, IS_DICT),
-        "CacheClusterEnabled": PropertyType(False, is_type(bool)),
-        "CacheClusterSize": PropertyType(False, IS_STR),
-        "CanarySetting": PropertyType(False, IS_DICT),
-        "ClientCertificateId": PropertyType(False, IS_STR),
-        "DeploymentId": PropertyType(True, IS_STR),
-        "Description": PropertyType(False, IS_STR),
-        "RestApiId": PropertyType(True, IS_STR),
-        "StageName": PropertyType(True, one_of(IS_STR, IS_DICT)),
-        "Tags": PropertyType(False, list_of(IS_DICT)),
-        "TracingEnabled": PropertyType(False, is_type(bool)),
-        "Variables": PropertyType(False, IS_DICT),
-        "MethodSettings": PropertyType(False, is_type(list)),
+        "AccessLogSetting": GeneratedProperty(),
+        "CacheClusterEnabled": GeneratedProperty(),
+        "CacheClusterSize": GeneratedProperty(),
+        "CanarySetting": GeneratedProperty(),
+        "ClientCertificateId": GeneratedProperty(),
+        "DeploymentId": GeneratedProperty(),
+        "Description": GeneratedProperty(),
+        "RestApiId": GeneratedProperty(),
+        "StageName": GeneratedProperty(),
+        "Tags": GeneratedProperty(),
+        "TracingEnabled": GeneratedProperty(),
+        "Variables": GeneratedProperty(),
+        "MethodSettings": GeneratedProperty(),
     }
 
     runtime_attrs = {"stage_name": lambda self: ref(self.logical_id)}
 
-    def update_deployment_ref(self, deployment_logical_id):  # type: ignore[no-untyped-def]
+    def update_deployment_ref(self, deployment_logical_id: str) -> None:
         self.DeploymentId = ref(deployment_logical_id)
 
 
 class ApiGatewayAccount(Resource):
     resource_type = "AWS::ApiGateway::Account"
-    property_types = {"CloudWatchRoleArn": PropertyType(False, one_of(IS_STR, IS_DICT))}
+    property_types = {
+        "CloudWatchRoleArn": GeneratedProperty(),
+    }
 
 
 class ApiGatewayDeployment(Resource):
@@ -80,21 +81,27 @@ class ApiGatewayDeployment(Resource):
 
     resource_type = "AWS::ApiGateway::Deployment"
     property_types = {
-        "Description": PropertyType(False, IS_STR),
-        "RestApiId": PropertyType(True, IS_STR),
-        "StageDescription": PropertyType(False, IS_DICT),
-        "StageName": PropertyType(False, IS_STR),
+        "Description": GeneratedProperty(),
+        "RestApiId": GeneratedProperty(),
+        "StageDescription": GeneratedProperty(),
+        "StageName": GeneratedProperty(),
     }
 
     runtime_attrs = {"deployment_id": lambda self: ref(self.logical_id)}
 
-    def make_auto_deployable(  # type: ignore[no-untyped-def]
-        self, stage, openapi_version=None, swagger=None, domain=None, redeploy_restapi_parameters=None
-    ):
+    def make_auto_deployable(
+        self,
+        stage: ApiGatewayStage,
+        openapi_version: Optional[Union[Dict[str, Any], str]] = None,
+        swagger: Optional[Dict[str, Any]] = None,
+        domain: Optional[Dict[str, Any]] = None,
+        redeploy_restapi_parameters: Optional[Any] = None,
+    ) -> None:
         """
         Sets up the resource such that it will trigger a re-deployment when Swagger changes
         or the openapi version changes or a domain resource changes.
 
+        :param stage: The ApiGatewayStage object which will be re-deployed
         :param swagger: Dictionary containing the Swagger definition of the API
         :param openapi_version: string containing value of OpenApiVersion flag in the template
         :param domain: Dictionary containing the custom domain configuration for the API
@@ -114,10 +121,7 @@ class ApiGatewayDeployment(Resource):
             hash_input.append(str(openapi_version))
         if domain:
             hash_input.append(json.dumps(domain))
-        if redeploy_restapi_parameters:
-            function_names = redeploy_restapi_parameters.get("function_names")
-        else:
-            function_names = None
+        function_names = redeploy_restapi_parameters.get("function_names") if redeploy_restapi_parameters else None
         # The deployment logical id is <api logicalId> + "Deployment"
         # The keyword "Deployment" is removed and all the function names associated with api is obtained
         if function_names and function_names.get(self.logical_id[:-10], None):
@@ -125,12 +129,12 @@ class ApiGatewayDeployment(Resource):
         data = self._X_HASH_DELIMITER.join(hash_input)
         generator = logical_id_generator.LogicalIdGenerator(self.logical_id, data)
         self.logical_id = generator.gen()
-        digest = generator.get_hash(length=40)  # type: ignore[no-untyped-call] # Get the full hash
+        digest = generator.get_hash(length=40)
         self.Description = "RestApi deployment id: {}".format(digest)
         stage.update_deployment_ref(self.logical_id)
 
 
-class ApiGatewayResponse(object):
+class ApiGatewayResponse:
     ResponseParameterProperties = ["Headers", "Paths", "QueryStrings"]
 
     def __init__(
@@ -142,7 +146,7 @@ class ApiGatewayResponse(object):
     ) -> None:
         if response_parameters:
             # response_parameters has been validated in ApiGenerator._add_gateway_responses()
-            for response_parameter_key in response_parameters.keys():
+            for response_parameter_key in response_parameters:
                 if response_parameter_key not in ApiGatewayResponse.ResponseParameterProperties:
                     raise InvalidResourceException(
                         api_logical_id, "Invalid gateway response parameter '{}'".format(response_parameter_key)
@@ -159,10 +163,10 @@ class ApiGatewayResponse(object):
         self.response_templates = response_templates or Py27Dict()
         self.status_code = status_code_str
 
-    def generate_swagger(self):  # type: ignore[no-untyped-def]
+    def generate_swagger(self) -> Py27Dict:
         # Applying Py27Dict here as this goes into swagger
         swagger = Py27Dict()
-        swagger["responseParameters"] = self._add_prefixes(self.response_parameters)  # type: ignore[no-untyped-call]
+        swagger["responseParameters"] = self._add_prefixes(self.response_parameters)
         swagger["responseTemplates"] = self.response_templates
 
         # Prevent "null" being written.
@@ -171,7 +175,7 @@ class ApiGatewayResponse(object):
 
         return swagger
 
-    def _add_prefixes(self, response_parameters):  # type: ignore[no-untyped-def]
+    def _add_prefixes(self, response_parameters: Dict[str, Any]) -> Dict[str, str]:
         GATEWAY_RESPONSE_PREFIX = "gatewayresponse."
         # applying Py27Dict as this is part of swagger
         prefixed_parameters = Py27Dict()
@@ -198,35 +202,43 @@ class ApiGatewayResponse(object):
 class ApiGatewayDomainName(Resource):
     resource_type = "AWS::ApiGateway::DomainName"
     property_types = {
-        "RegionalCertificateArn": PropertyType(False, IS_STR),
-        "DomainName": PropertyType(True, IS_STR),
-        "EndpointConfiguration": PropertyType(False, IS_DICT),
-        "MutualTlsAuthentication": PropertyType(False, IS_DICT),
-        "SecurityPolicy": PropertyType(False, IS_STR),
-        "CertificateArn": PropertyType(False, IS_STR),
-        "OwnershipVerificationCertificateArn": PropertyType(False, IS_STR),
+        "RegionalCertificateArn": GeneratedProperty(),
+        "DomainName": GeneratedProperty(),
+        "EndpointConfiguration": GeneratedProperty(),
+        "MutualTlsAuthentication": GeneratedProperty(),
+        "SecurityPolicy": GeneratedProperty(),
+        "CertificateArn": GeneratedProperty(),
+        "OwnershipVerificationCertificateArn": GeneratedProperty(),
     }
 
+    RegionalCertificateArn: Optional[PassThrough]
+    DomainName: PassThrough
+    EndpointConfiguration: Optional[PassThrough]
+    MutualTlsAuthentication: Optional[Dict[str, Any]]
+    SecurityPolicy: Optional[PassThrough]
+    CertificateArn: Optional[PassThrough]
+    OwnershipVerificationCertificateArn: Optional[PassThrough]
+
 
 class ApiGatewayBasePathMapping(Resource):
     resource_type = "AWS::ApiGateway::BasePathMapping"
     property_types = {
-        "BasePath": PropertyType(False, IS_STR),
-        "DomainName": PropertyType(True, IS_STR),
-        "RestApiId": PropertyType(False, IS_STR),
-        "Stage": PropertyType(False, IS_STR),
+        "BasePath": GeneratedProperty(),
+        "DomainName": GeneratedProperty(),
+        "RestApiId": GeneratedProperty(),
+        "Stage": GeneratedProperty(),
     }
 
 
 class ApiGatewayUsagePlan(Resource):
     resource_type = "AWS::ApiGateway::UsagePlan"
     property_types = {
-        "ApiStages": PropertyType(False, is_type(list)),
-        "Description": PropertyType(False, IS_STR),
-        "Quota": PropertyType(False, IS_DICT),
-        "Tags": PropertyType(False, list_of(dict)),
-        "Throttle": PropertyType(False, IS_DICT),
-        "UsagePlanName": PropertyType(False, IS_STR),
+        "ApiStages": GeneratedProperty(),
+        "Description": GeneratedProperty(),
+        "Quota": GeneratedProperty(),
+        "Tags": GeneratedProperty(),
+        "Throttle": GeneratedProperty(),
+        "UsagePlanName": GeneratedProperty(),
     }
     runtime_attrs = {"usage_plan_id": lambda self: ref(self.logical_id)}
 
@@ -234,41 +246,42 @@ class ApiGatewayUsagePlan(Resource):
 class ApiGatewayUsagePlanKey(Resource):
     resource_type = "AWS::ApiGateway::UsagePlanKey"
     property_types = {
-        "KeyId": PropertyType(True, IS_STR),
-        "KeyType": PropertyType(True, IS_STR),
-        "UsagePlanId": PropertyType(True, IS_STR),
+        "KeyId": GeneratedProperty(),
+        "KeyType": GeneratedProperty(),
+        "UsagePlanId": GeneratedProperty(),
     }
 
 
 class ApiGatewayApiKey(Resource):
     resource_type = "AWS::ApiGateway::ApiKey"
     property_types = {
-        "CustomerId": PropertyType(False, IS_STR),
-        "Description": PropertyType(False, IS_STR),
-        "Enabled": PropertyType(False, is_type(bool)),
-        "GenerateDistinctId": PropertyType(False, is_type(bool)),
-        "Name": PropertyType(False, IS_STR),
-        "StageKeys": PropertyType(False, is_type(list)),
-        "Value": PropertyType(False, IS_STR),
+        "CustomerId": GeneratedProperty(),
+        "Description": GeneratedProperty(),
+        "Enabled": GeneratedProperty(),
+        "GenerateDistinctId": GeneratedProperty(),
+        "Name": GeneratedProperty(),
+        "StageKeys": GeneratedProperty(),
+        "Value": GeneratedProperty(),
     }
 
     runtime_attrs = {"api_key_id": lambda self: ref(self.logical_id)}
 
 
-class ApiGatewayAuthorizer(object):
+class ApiGatewayAuthorizer:
     _VALID_FUNCTION_PAYLOAD_TYPES = [None, "TOKEN", "REQUEST"]
 
-    def __init__(  # type: ignore[no-untyped-def]
+    def __init__(  # type: ignore[no-untyped-def]# noqa: too-many-arguments
         self,
         api_logical_id=None,
         name=None,
         user_pool_arn=None,
         function_arn=None,
         identity=None,
-        function_payload_type=None,
+        function_payload_type: Optional[str] = None,
         function_invoke_role=None,
         is_aws_iam_authorizer=False,
         authorization_scopes=None,
+        disable_function_default_permissions=False,
     ):
         if authorization_scopes is None:
             authorization_scopes = []
@@ -282,6 +295,7 @@ class ApiGatewayAuthorizer(object):
         self.function_invoke_role = function_invoke_role
         self.is_aws_iam_authorizer = is_aws_iam_authorizer
         self.authorization_scopes = authorization_scopes
+        self.disable_function_default_permissions = disable_function_default_permissions
 
         if function_payload_type not in ApiGatewayAuthorizer._VALID_FUNCTION_PAYLOAD_TYPES:
             raise InvalidResourceException(
@@ -296,8 +310,15 @@ class ApiGatewayAuthorizer(object):
                 "of Headers, QueryStrings, StageVariables, or Context.",
             )
 
-        if authorization_scopes is not None and not isinstance(authorization_scopes, list):
-            raise InvalidResourceException(api_logical_id, "AuthorizationScopes must be a list.")
+        if authorization_scopes is not None:
+            sam_expect(authorization_scopes, api_logical_id, f"Authorizers.{name}.AuthorizationScopes").to_be_a_list()
+
+        if disable_function_default_permissions is not None:
+            sam_expect(
+                disable_function_default_permissions,
+                api_logical_id,
+                f"Authorizers.{name}.DisableFunctionDefaultPermissions",
+            ).to_be_a_bool()
 
     def _is_missing_identity_source(self, identity: Dict[str, Any]) -> bool:
         if not identity:
@@ -325,35 +346,35 @@ class ApiGatewayAuthorizer(object):
         # If we can resolve ttl, attempt to see if things are valid
         return ttl_int > 0 and required_properties_missing
 
-    def generate_swagger(self):  # type: ignore[no-untyped-def]
-        authorizer_type = self._get_type()  # type: ignore[no-untyped-call]
+    def generate_swagger(self) -> Py27Dict:
+        authorizer_type = self._get_type()
         APIGATEWAY_AUTHORIZER_KEY = "x-amazon-apigateway-authorizer"
         swagger = Py27Dict()
         swagger["type"] = "apiKey"
-        swagger["name"] = self._get_swagger_header_name()  # type: ignore[no-untyped-call]
+        swagger["name"] = self._get_swagger_header_name()
         swagger["in"] = "header"
-        swagger["x-amazon-apigateway-authtype"] = self._get_swagger_authtype()  # type: ignore[no-untyped-call]
+        swagger["x-amazon-apigateway-authtype"] = self._get_swagger_authtype()
 
         if authorizer_type == "COGNITO_USER_POOLS":
             authorizer_dict = Py27Dict()
-            authorizer_dict["type"] = self._get_swagger_authorizer_type()  # type: ignore[no-untyped-call]
-            authorizer_dict["providerARNs"] = self._get_user_pool_arn_array()  # type: ignore[no-untyped-call]
+            authorizer_dict["type"] = self._get_swagger_authorizer_type()
+            authorizer_dict["providerARNs"] = self._get_user_pool_arn_array()
             swagger[APIGATEWAY_AUTHORIZER_KEY] = authorizer_dict
 
         elif authorizer_type == "LAMBDA":
-            swagger[APIGATEWAY_AUTHORIZER_KEY] = Py27Dict({"type": self._get_swagger_authorizer_type()})  # type: ignore[no-untyped-call, no-untyped-call]
+            swagger[APIGATEWAY_AUTHORIZER_KEY] = Py27Dict({"type": self._get_swagger_authorizer_type()})
             partition = ArnGenerator.get_partition_name()
             resource = "lambda:path/2015-03-31/functions/${__FunctionArn__}/invocations"
             authorizer_uri = fnSub(
-                ArnGenerator.generate_arn(  # type: ignore[no-untyped-call]
+                ArnGenerator.generate_arn(
                     partition=partition, service="apigateway", resource=resource, include_account_id=False
                 ),
                 {"__FunctionArn__": self.function_arn},
             )
 
             swagger[APIGATEWAY_AUTHORIZER_KEY]["authorizerUri"] = authorizer_uri
-            reauthorize_every = self._get_reauthorize_every()  # type: ignore[no-untyped-call]
-            function_invoke_role = self._get_function_invoke_role()  # type: ignore[no-untyped-call]
+            reauthorize_every = self._get_reauthorize_every()
+            function_invoke_role = self._get_function_invoke_role()
 
             if reauthorize_every is not None:
                 swagger[APIGATEWAY_AUTHORIZER_KEY]["authorizerResultTtlInSeconds"] = reauthorize_every
@@ -361,30 +382,30 @@ class ApiGatewayAuthorizer(object):
             if function_invoke_role:
                 swagger[APIGATEWAY_AUTHORIZER_KEY]["authorizerCredentials"] = function_invoke_role
 
-            if self._get_function_payload_type() == "REQUEST":  # type: ignore[no-untyped-call]
+            if self._get_function_payload_type() == "REQUEST":
                 identity_source = self._get_identity_source()
                 if identity_source:
                     swagger[APIGATEWAY_AUTHORIZER_KEY]["identitySource"] = self._get_identity_source()
 
         # Authorizer Validation Expression is only allowed on COGNITO_USER_POOLS and LAMBDA_TOKEN
-        is_lambda_token_authorizer = authorizer_type == "LAMBDA" and self._get_function_payload_type() == "TOKEN"  # type: ignore[no-untyped-call]
+        is_lambda_token_authorizer = authorizer_type == "LAMBDA" and self._get_function_payload_type() == "TOKEN"
 
         if authorizer_type == "COGNITO_USER_POOLS" or is_lambda_token_authorizer:
-            identity_validation_expression = self._get_identity_validation_expression()  # type: ignore[no-untyped-call]
+            identity_validation_expression = self._get_identity_validation_expression()
 
             if identity_validation_expression:
                 swagger[APIGATEWAY_AUTHORIZER_KEY]["identityValidationExpression"] = identity_validation_expression
 
         return swagger
 
-    def _get_identity_validation_expression(self):  # type: ignore[no-untyped-def]
+    def _get_identity_validation_expression(self) -> Optional[PassThrough]:
         return self.identity and self.identity.get("ValidationExpression")
 
     @staticmethod
     def _build_identity_source_item(item_prefix: str, prop_value: str) -> str:
         item = item_prefix + prop_value
         if isinstance(prop_value, Py27UniStr):
-            item = Py27UniStr(item)
+            return Py27UniStr(item)
         return item
 
     def _build_identity_source_item_array(self, prop_key: str, item_prefix: str) -> List[str]:
@@ -413,23 +434,23 @@ class ApiGatewayAuthorizer(object):
         identity_source = ", ".join(identity_source_array)
         if any(isinstance(i, Py27UniStr) for i in identity_source_array):
             # Convert identity_source to Py27UniStr if any part of it is Py27UniStr
-            identity_source = Py27UniStr(identity_source)
+            return Py27UniStr(identity_source)
 
         return identity_source
 
-    def _get_user_pool_arn_array(self):  # type: ignore[no-untyped-def]
+    def _get_user_pool_arn_array(self) -> List[PassThrough]:
         return self.user_pool_arn if isinstance(self.user_pool_arn, list) else [self.user_pool_arn]
 
-    def _get_swagger_header_name(self):  # type: ignore[no-untyped-def]
-        authorizer_type = self._get_type()  # type: ignore[no-untyped-call]
-        payload_type = self._get_function_payload_type()  # type: ignore[no-untyped-call]
+    def _get_swagger_header_name(self) -> Optional[str]:
+        authorizer_type = self._get_type()
+        payload_type = self._get_function_payload_type()
 
         if authorizer_type == "LAMBDA" and payload_type == "REQUEST":
             return "Unused"
 
-        return self._get_identity_header()  # type: ignore[no-untyped-call]
+        return self._get_identity_header()
 
-    def _get_type(self):  # type: ignore[no-untyped-def]
+    def _get_type(self) -> str:
         if self.is_aws_iam_authorizer:
             return "AWS_IAM"
 
@@ -438,7 +459,7 @@ class ApiGatewayAuthorizer(object):
 
         return "LAMBDA"
 
-    def _get_identity_header(self):  # type: ignore[no-untyped-def]
+    def _get_identity_header(self) -> Optional[str]:
         if self.identity and not isinstance(self.identity, dict):
             raise InvalidResourceException(
                 self.api_logical_id,
@@ -451,20 +472,20 @@ class ApiGatewayAuthorizer(object):
 
         return self.identity.get("Header")
 
-    def _get_reauthorize_every(self):  # type: ignore[no-untyped-def]
+    def _get_reauthorize_every(self) -> Optional[PassThrough]:
         if not self.identity:
             return None
 
         return self.identity.get("ReauthorizeEvery")
 
-    def _get_function_invoke_role(self):  # type: ignore[no-untyped-def]
+    def _get_function_invoke_role(self) -> Optional[PassThrough]:
         if not self.function_invoke_role or self.function_invoke_role == "NONE":
             return None
 
         return self.function_invoke_role
 
-    def _get_swagger_authtype(self):  # type: ignore[no-untyped-def]
-        authorizer_type = self._get_type()  # type: ignore[no-untyped-call]
+    def _get_swagger_authtype(self) -> str:
+        authorizer_type = self._get_type()
         if authorizer_type == "AWS_IAM":
             return "awsSigv4"
 
@@ -473,19 +494,21 @@ class ApiGatewayAuthorizer(object):
 
         return "custom"
 
-    def _get_function_payload_type(self):  # type: ignore[no-untyped-def]
+    def _get_function_payload_type(self) -> str:
         return "TOKEN" if not self.function_payload_type else self.function_payload_type
 
-    def _get_swagger_authorizer_type(self):  # type: ignore[no-untyped-def]
-        authorizer_type = self._get_type()  # type: ignore[no-untyped-call]
+    def _get_swagger_authorizer_type(self) -> Optional[str]:
+        authorizer_type = self._get_type()
 
         if authorizer_type == "COGNITO_USER_POOLS":
             return "cognito_user_pools"
 
-        payload_type = self._get_function_payload_type()  # type: ignore[no-untyped-call]
+        payload_type = self._get_function_payload_type()
 
         if payload_type == "REQUEST":
             return "request"
 
         if payload_type == "TOKEN":
             return "token"
+
+        return None  # should we raise validation error here?
