@@ -5,28 +5,27 @@ from typing import Any, Dict, List, Optional, Union, cast
 
 from samtranslator.intrinsics.resolver import IntrinsicsResolver
 from samtranslator.metrics.method_decorator import cw_timer
-from samtranslator.model import ResourceMacro, PropertyType
+from samtranslator.model import PropertyType, ResourceMacro
+from samtranslator.model.cognito import CognitoUserPool
+from samtranslator.model.eventbridge_utils import EventBridgeRuleUtils
+from samtranslator.model.events import EventsRule
 from samtranslator.model.eventsources import FUNCTION_EVETSOURCE_METRIC_PREFIX
-from samtranslator.model.types import IS_DICT, is_type, list_of, dict_of, one_of, IS_STR
-from samtranslator.model.intrinsics import is_intrinsic, ref, fnGetAtt, fnSub, make_shorthand, make_conditional
-from samtranslator.model.tags.resource_tagging import get_tag_list
-
+from samtranslator.model.eventsources.pull import SQS
+from samtranslator.model.exceptions import InvalidDocumentException, InvalidEventException, InvalidResourceException
+from samtranslator.model.intrinsics import fnGetAtt, fnSub, is_intrinsic, make_conditional, make_shorthand, ref
+from samtranslator.model.iot import IotTopicRule
+from samtranslator.model.lambda_ import LambdaPermission
 from samtranslator.model.s3 import S3Bucket
 from samtranslator.model.sns import SNSSubscription
-from samtranslator.model.lambda_ import LambdaPermission
-from samtranslator.model.events import EventsRule
-from samtranslator.model.eventsources.pull import SQS
-from samtranslator.model.sqs import SQSQueue, SQSQueuePolicy, SQSQueuePolicies
-from samtranslator.model.eventbridge_utils import EventBridgeRuleUtils
-from samtranslator.model.iot import IotTopicRule
-from samtranslator.model.cognito import CognitoUserPool
-from samtranslator.schema.common import PassThrough
+from samtranslator.model.sqs import SQSQueue, SQSQueuePolicies, SQSQueuePolicy
+from samtranslator.model.tags.resource_tagging import get_tag_list
+from samtranslator.model.types import IS_DICT, IS_STR, PassThrough, dict_of, is_type, list_of, one_of
+from samtranslator.open_api.open_api import OpenApiEditor
+from samtranslator.swagger.swagger import SwaggerEditor
 from samtranslator.translator import logical_id_generator
 from samtranslator.translator.arn_generator import ArnGenerator
-from samtranslator.model.exceptions import InvalidEventException, InvalidResourceException, InvalidDocumentException
-from samtranslator.swagger.swagger import SwaggerEditor
-from samtranslator.open_api.open_api import OpenApiEditor
 from samtranslator.utils.py27hash_fix import Py27Dict, Py27UniStr
+from samtranslator.utils.utils import InvalidValueType, dict_deep_get
 from samtranslator.validator.value_validator import sam_expect
 
 CONDITION = "Condition"
@@ -61,7 +60,7 @@ class PushEventSource(ResourceMacro, metaclass=ABCMeta):
     principal: str = None  # type: ignore
     relative_id: str  # overriding the Optional[str]: for event, relative id is not None
 
-    def _construct_permission(  # type: ignore[no-untyped-def]
+    def _construct_permission(  # type: ignore[no-untyped-def] # noqa: too-many-arguments
         self, function, source_arn=None, source_account=None, suffix="", event_source_token=None, prefix=None
     ):
         """Constructs the Lambda Permission resource allowing the source service to invoke the function this event
@@ -441,7 +440,6 @@ class S3(PushEventSource):
 
         event_mappings = []
         for event_type in event_types:
-
             lambda_event = copy.deepcopy(base_event_mapping)
             lambda_event["Event"] = event_type
             if CONDITION in function.resource_attributes:
@@ -565,7 +563,7 @@ class SNS(PushEventSource):
         resources.append(subscription)
         return resources
 
-    def _inject_subscription(
+    def _inject_subscription(  # noqa: too-many-arguments
         self,
         protocol: str,
         endpoint: str,
@@ -600,7 +598,7 @@ class SNS(PushEventSource):
         )
         event_source.Queue = queue_arn
         event_source.BatchSize = batch_size or 10
-        event_source.Enabled = enabled or True
+        event_source.Enabled = True
         return event_source.to_cloudformation(function=function, role=role)
 
     def _inject_sqs_queue_policy(self, topic_arn, queue_arn, queue_url, function, logical_id=None):  # type: ignore[no-untyped-def]
@@ -637,49 +635,49 @@ class Api(PushEventSource):
     RequestModel: Optional[Dict[str, Any]]
     RequestParameters: Optional[List[Any]]
 
-    def resources_to_link(self, resources):  # type: ignore[no-untyped-def]
+    def resources_to_link(self, resources: Dict[str, Any]) -> Dict[str, Any]:
         """
         If this API Event Source refers to an explicit API resource, resolve the reference and grab
         necessary data from the explicit API
         """
+        return self.resources_to_link_for_rest_api(resources, self.relative_id, self.RestApiId)
 
+    @staticmethod
+    def resources_to_link_for_rest_api(
+        resources: Dict[str, Any], relative_id: str, raw_rest_api_id: Optional[Any]
+    ) -> Dict[str, Any]:
         # If RestApiId is a resource in the same template, then we try find the StageName by following the reference
         # Otherwise we default to a wildcard. This stage name is solely used to construct the permission to
         # allow this stage to invoke the Lambda function. If we are unable to resolve the stage name, we will
         # simply permit all stages to invoke this Lambda function
         # This hack is necessary because customers could use !ImportValue, !Ref or other intrinsic functions which
         # can be sometimes impossible to resolve (ie. when it has cross-stack references)
-        permitted_stage = "*"
         stage_suffix = "AllStages"
-        explicit_api = None
-        rest_api_id = self.get_rest_api_id_string(self.RestApiId)
+        explicit_api_resource_properties = None
+        rest_api_id = Api.get_rest_api_id_string(raw_rest_api_id)
         if isinstance(rest_api_id, str):
+            rest_api_resource = sam_expect(
+                resources.get(rest_api_id), relative_id, "RestApiId", is_sam_event=True
+            ).to_be_a_map("RestApiId property of Api event must reference a valid resource in the same template.")
 
-            if (
-                rest_api_id in resources
-                and "Properties" in resources[rest_api_id]
-                and "StageName" in resources[rest_api_id]["Properties"]
-            ):
-
-                explicit_api = resources[rest_api_id]["Properties"]
-                permitted_stage = explicit_api["StageName"]
-
-                # Stage could be a intrinsic, in which case leave the suffix to default value
-                if isinstance(permitted_stage, str):
-                    if not permitted_stage:
-                        raise InvalidResourceException(rest_api_id, "StageName cannot be empty.")
-                    stage_suffix = permitted_stage
-                else:
-                    stage_suffix = "Stage"  # type: ignore[unreachable]
+            explicit_api_resource_properties = sam_expect(
+                rest_api_resource.get("Properties", {}), rest_api_id, "Properties", is_resource_attribute=True
+            ).to_be_a_map()
+            permitted_stage = explicit_api_resource_properties.get("StageName")
 
+            # Stage could be an intrinsic, in which case leave the suffix to default value
+            if isinstance(permitted_stage, str):
+                if not permitted_stage:
+                    raise InvalidResourceException(rest_api_id, "StageName cannot be empty.")
+                stage_suffix = permitted_stage
             else:
-                # RestApiId is a string, not an intrinsic, but we did not find a valid API resource for this ID
-                raise InvalidEventException(
-                    self.relative_id,
-                    "RestApiId property of Api event must reference a valid resource in the same template.",
-                )
+                stage_suffix = "Stage"
 
-        return {"explicit_api": explicit_api, "api_id": rest_api_id, "explicit_api_stage": {"suffix": stage_suffix}}
+        return {
+            "explicit_api": explicit_api_resource_properties,
+            "api_id": rest_api_id,
+            "explicit_api_stage": {"suffix": stage_suffix},
+        }
 
     @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
@@ -708,7 +706,7 @@ class Api(PushEventSource):
 
         explicit_api = kwargs["explicit_api"]
         api_id = kwargs["api_id"]
-        if explicit_api.get("__MANAGE_SWAGGER"):
+        if explicit_api.get("__MANAGE_SWAGGER") or explicit_api.get("MergeDefinitions"):
             self._add_swagger_integration(explicit_api, api_id, function, intrinsics_resolver)  # type: ignore[no-untyped-call]
 
         return resources
@@ -743,24 +741,30 @@ class Api(PushEventSource):
         api_id = self.RestApiId
 
         # RestApiId can be a simple string or intrinsic function like !Ref. Using Fn::Sub will handle both cases
-        resource = "${__ApiId__}/" + "${__Stage__}/" + method + path
+        resource = f"${{__ApiId__}}/${{__Stage__}}/{method}{path}"
         partition = ArnGenerator.get_partition_name()
         source_arn = fnSub(
-            ArnGenerator.generate_arn(partition=partition, service="execute-api", resource=resource),  # type: ignore[no-untyped-call]
+            ArnGenerator.generate_arn(partition=partition, service="execute-api", resource=resource),
             {"__ApiId__": api_id, "__Stage__": stage},
         )
 
         return self._construct_permission(resources_to_link["function"], source_arn=source_arn, suffix=suffix)  # type: ignore[no-untyped-call]
 
-    def _add_swagger_integration(self, api, api_id, function, intrinsics_resolver):  # type: ignore[no-untyped-def]
-        # pylint: disable=duplicate-code
+    def _add_swagger_integration(  # type: ignore[no-untyped-def] # noqa: too-many-statements
+        self, api, api_id, function, intrinsics_resolver
+    ):
         """Adds the path and method for this Api event source to the Swagger body for the provided RestApi.
 
         :param model.apigateway.ApiGatewayRestApi rest_api: the RestApi to which the path and method should be added.
         """
         swagger_body = api.get("DefinitionBody")
+        merge_definitions = api.get("MergeDefinitions")
         if swagger_body is None:
             return
+        if merge_definitions:
+            # Use a skeleton swagger body for API event source to make sure the generated definition body
+            # is unaffected by the inline/customer defined DefinitionBody
+            swagger_body = SwaggerEditor.gen_skeleton()
 
         partition = ArnGenerator.get_partition_name()
         uri = _build_apigw_integration_uri(function, partition)  # type: ignore[no-untyped-call]
@@ -845,7 +849,6 @@ class Api(PushEventSource):
 
                 # Checking if any of the fields are defined as it can be false we are checking if the field are not None
                 if validate_body is not None or validate_parameters is not None:
-
                     # as we are setting two different fields we are here setting as default False
                     # In case one of them are not defined
                     validate_body = False if validate_body is None else validate_body
@@ -870,14 +873,11 @@ class Api(PushEventSource):
                     )
 
         if self.RequestParameters:
-
             default_value = {"Required": False, "Caching": False}
 
             parameters = []
             for parameter in self.RequestParameters:
-
                 if isinstance(parameter, dict):
-
                     parameter_name, parameter_value = next(iter(parameter.items()))
 
                     if not re.match(r"method\.request\.(querystring|path|header)\.", parameter_name):
@@ -889,7 +889,7 @@ class Api(PushEventSource):
                         )
 
                     if not isinstance(parameter_value, dict) or not all(
-                        key in REQUEST_PARAMETER_PROPERTIES for key in parameter_value.keys()
+                        key in REQUEST_PARAMETER_PROPERTIES for key in parameter_value
                     ):
                         raise InvalidEventException(
                             self.relative_id,
@@ -927,7 +927,38 @@ class Api(PushEventSource):
                 path=self.Path, method_name=self.Method, request_parameters=parameters
             )
 
-        api["DefinitionBody"] = editor.swagger
+        if merge_definitions:
+            api["DefinitionBody"] = self._get_merged_definitions(api_id, api["DefinitionBody"], editor.swagger)
+        else:
+            api["DefinitionBody"] = editor.swagger
+
+    def _get_merged_definitions(
+        self, api_id: str, source_definition_body: Dict[str, Any], dest_definition_body: Dict[str, Any]
+    ) -> Dict[str, Any]:
+        """
+        Merge SAM generated swagger definition(dest_definition_body) into inline DefinitionBody(source_definition_body):
+        - for a conflicting key, use SAM generated value
+        - otherwise include key-value pairs from both definitions
+        """
+        merged_definition_body = source_definition_body.copy()
+        source_body_paths = merged_definition_body.get("paths", {})
+
+        try:
+            path_method_body = dict_deep_get(source_body_paths, [self.Path, self.Method]) or {}
+        except InvalidValueType as e:
+            raise InvalidResourceException(api_id, f"Property 'DefinitionBody' is invalid: {str(e)}") from e
+
+        sam_expect(path_method_body, api_id, f"DefinitionBody.paths.{self.Path}.{self.Method}").to_be_a_map()
+
+        generated_path_method_body = dest_definition_body["paths"][self.Path][self.Method]
+        # this guarantees that the merged definition use SAM generated value for a conflicting key
+        merged_path_method_body = {**path_method_body, **generated_path_method_body}
+
+        if self.Path not in source_body_paths:
+            source_body_paths[self.Path] = {self.Method: merged_path_method_body}
+        source_body_paths[self.Path][self.Method] = merged_path_method_body
+
+        return merged_definition_body
 
     @staticmethod
     def get_rest_api_id_string(rest_api_id: Any) -> Any:
@@ -941,7 +972,7 @@ class Api(PushEventSource):
         return rest_api_id["Ref"] if isinstance(rest_api_id, dict) and "Ref" in rest_api_id else rest_api_id
 
     @staticmethod
-    def add_auth_to_swagger(
+    def add_auth_to_swagger(  # noqa: too-many-arguments
         event_auth: Dict[str, Any],
         api: Dict[str, Any],
         api_id: str,
@@ -1063,7 +1094,7 @@ class IoTRule(PushEventSource):
 
         partition = ArnGenerator.get_partition_name()
         source_arn = fnSub(
-            ArnGenerator.generate_arn(partition=partition, service="iot", resource=resource),  # type: ignore[no-untyped-call]
+            ArnGenerator.generate_arn(partition=partition, service="iot", resource=resource),
             {"RuleName": ref(self.logical_id)},
         )
         source_account = fnSub("${AWS::AccountId}")
@@ -1290,14 +1321,14 @@ class HttpApi(PushEventSource):
             # Case where default exists for this function, and so the permissions for that will apply here as well
             # This can save us several CFN resources (not duplicating permissions)
             return None
-        else:
-            path = OpenApiEditor.get_path_without_trailing_slash(path)  # type: ignore[no-untyped-call]
+        path = OpenApiEditor.get_path_without_trailing_slash(path)  # type: ignore[no-untyped-call]
 
         # Handle case where Method is already the ANY ApiGateway extension
-        if self._method.lower() == "any" or self._method.lower() == OpenApiEditor._X_ANY_METHOD:
-            method = "*"
-        else:
-            method = self._method.upper()
+        method = (
+            "*"
+            if self._method.lower() == "any" or self._method.lower() == OpenApiEditor._X_ANY_METHOD
+            else self._method.upper()
+        )
 
         api_id = self.ApiId
 
@@ -1312,7 +1343,7 @@ class HttpApi(PushEventSource):
 
         # ApiId can be a simple string or intrinsic function like !Ref. Using Fn::Sub will handle both cases
         source_arn = fnSub(
-            ArnGenerator.generate_arn(partition="${AWS::Partition}", service="execute-api", resource=resource),  # type: ignore[no-untyped-call]
+            ArnGenerator.generate_arn(partition="${AWS::Partition}", service="execute-api", resource=resource),
             {"__ApiId__": api_id, "__Stage__": stage},
         )
 
