@@ -1,6 +1,7 @@
 from abc import ABC, abstractmethod
+from collections import defaultdict
 from enum import Enum
-from typing import List, Optional, Sequence, Union
+from typing import Any, Dict, List, Optional, Sequence, Union
 
 
 class ExpectedType(Enum):
@@ -8,6 +9,7 @@ class ExpectedType(Enum):
     LIST = ("list", list)
     STRING = ("string", str)
     INTEGER = ("integer", int)
+    BOOLEAN = ("boolean", bool)
 
 
 class ExceptionWithMessage(ABC, Exception):
@@ -16,12 +18,17 @@ class ExceptionWithMessage(ABC, Exception):
     def message(self) -> str:
         """Return the exception message."""
 
+    @property
+    def metadata(self) -> Optional[Dict[str, Any]]:
+        """Return the exception metadata."""
+
 
 class InvalidDocumentException(ExceptionWithMessage):
     """Exception raised when the given document is invalid and cannot be transformed.
 
     Attributes:
         message -- explanation of the error
+        metadata -- a dictionary of metadata (key, value pair)
         causes -- list of errors which caused this document to be invalid
     """
 
@@ -37,6 +44,17 @@ class InvalidDocumentException(ExceptionWithMessage):
             len(self.causes)
         )
 
+    @property
+    def metadata(self) -> Dict[str, List[Any]]:
+        # Merge metadata in each exception to one single metadata dictionary
+        metadata_dict = defaultdict(list)
+        for cause in self.causes:
+            if not cause.metadata:
+                continue
+            for k, v in cause.metadata.items():
+                metadata_dict[k].append(v)
+        return metadata_dict
+
     @property
     def causes(self) -> Sequence[ExceptionWithMessage]:
         return self._causes
@@ -48,13 +66,13 @@ class DuplicateLogicalIdException(ExceptionWithMessage):
         message -- explanation of the error
     """
 
-    def __init__(self, logical_id: str, duplicate_id: str, type: str) -> None:
+    def __init__(self, logical_id: str, duplicate_id: str, resource_type: str) -> None:
         self._logical_id = logical_id
         self._duplicate_id = duplicate_id
-        self._type = type
+        self._type = resource_type
 
     @property
-    def message(self):  # type: ignore[no-untyped-def]
+    def message(self) -> str:
         return (
             "Transforming resource with id [{logical_id}] attempts to create a new"
             ' resource with id [{duplicate_id}] and type "{type}". A resource with that id already'
@@ -86,9 +104,12 @@ class InvalidResourceException(ExceptionWithMessage):
         message -- explanation of the error
     """
 
-    def __init__(self, logical_id: Union[str, List[str]], message: str) -> None:
+    def __init__(
+        self, logical_id: Union[str, List[str]], message: str, metadata: Optional[Dict[str, Any]] = None
+    ) -> None:
         self._logical_id = logical_id
         self._message = message
+        self._metadata = metadata
 
     def __lt__(self, other):  # type: ignore[no-untyped-def]
         return self._logical_id < other._logical_id
@@ -97,26 +118,49 @@ class InvalidResourceException(ExceptionWithMessage):
     def message(self) -> str:
         return "Resource with id [{}] is invalid. {}".format(self._logical_id, self._message)
 
+    @property
+    def metadata(self) -> Optional[Dict[str, Any]]:
+        return self._metadata
+
 
 class InvalidResourcePropertyTypeException(InvalidResourceException):
     def __init__(
         self,
         logical_id: str,
-        property_identifier: str,
+        key_path: str,
         expected_type: Optional[ExpectedType],
         message: Optional[str] = None,
     ) -> None:
-        message = message or self._default_message(property_identifier, expected_type)
+        message = message or self._default_message(key_path, expected_type)
         super().__init__(logical_id, message)
 
-        self.property_identifier = property_identifier
+        self.key_path = key_path
 
     @staticmethod
-    def _default_message(property_identifier: str, expected_type: Optional[ExpectedType]) -> str:
+    def _default_message(key_path: str, expected_type: Optional[ExpectedType]) -> str:
         if expected_type:
             type_description, _ = expected_type.value
-            return f"Property '{property_identifier}' should be a {type_description}."
-        return f"Type of property '{property_identifier}' is invalid."
+            return f"Property '{key_path}' should be a {type_description}."
+        return f"Type of property '{key_path}' is invalid."
+
+
+class InvalidResourceAttributeTypeException(InvalidResourceException):
+    def __init__(
+        self,
+        logical_id: str,
+        key_path: str,
+        expected_type: Optional[ExpectedType],
+        message: Optional[str] = None,
+    ) -> None:
+        message = message or self._default_message(logical_id, key_path, expected_type)
+        super().__init__(logical_id, message)
+
+    @staticmethod
+    def _default_message(logical_id: str, key_path: str, expected_type: Optional[ExpectedType]) -> str:
+        if expected_type:
+            type_description, _ = expected_type.value
+            return f"Attribute '{key_path}' should be a {type_description}."
+        return f"Type of attribute '{key_path}' is invalid."
 
 
 class InvalidEventException(ExceptionWithMessage):
