@@ -1,4 +1,8 @@
-from typing import Any, Dict, Iterable, List, Union, Optional
+from typing import Any, Dict, Iterable, List, Optional, Union
+
+MIN_NUM_CONDITIONS_TO_COMBINE = 2
+_NUM_ARGUMENTS_REQUIRED_IN_IF = 3
+_NUM_ARGUMENTS_REQUIRED_IN_GETATT = 2
 
 
 def fnGetAtt(logical_name: str, attribute_name: str) -> Dict[str, List[str]]:
@@ -47,14 +51,12 @@ def make_condition_or_list(conditions_list: Iterable[Any]) -> List[Dict[str, Any
 
 def make_or_condition(conditions_list: Iterable[Any]) -> Dict[str, List[Dict[str, Any]]]:
     or_list = make_condition_or_list(conditions_list)
-    condition = fnOr(or_list)
-    return condition
+    return fnOr(or_list)
 
 
 def make_and_condition(conditions_list: Iterable[Any]) -> Dict[str, List[Dict[str, Any]]]:
     and_list = make_condition_or_list(conditions_list)
-    condition = fnAnd(and_list)
-    return condition
+    return fnAnd(and_list)
 
 
 def calculate_number_of_conditions(conditions_length: int, max_conditions: int) -> int:
@@ -72,8 +74,7 @@ def calculate_number_of_conditions(conditions_length: int, max_conditions: int)
     :param int max_conditions: maximum number of conditions that can be put in an Fn::Or statement
     :return: the number (int) of necessary additional conditions.
     """
-    num_conditions = 1 + (conditions_length - 2) // (max_conditions - 1)
-    return num_conditions
+    return 1 + (conditions_length - 2) // (max_conditions - 1)
 
 
 def make_combined_condition(
@@ -88,8 +89,8 @@ def make_combined_condition(
     :param string condition_name: base name desired for new condition
     :return: dictionary of condition_name: condition_value
     """
-    if len(conditions_list) < 2:
-        # Can't make a condition if <2 conditions provided.
+    if len(conditions_list) < MIN_NUM_CONDITIONS_TO_COMBINE:
+        # Can't make a condition not enough conditions are provided.
         return None
 
     # Total number of conditions allows in an Fn::Or statement. See docs:
@@ -145,7 +146,6 @@ def is_intrinsic(_input: Any) -> bool:
     """
 
     if _input is not None and isinstance(_input, dict) and len(_input) == 1:
-
         key: str = list(_input.keys())[0]
         return key == "Ref" or key == "Condition" or key.startswith("Fn::")
 
@@ -182,8 +182,8 @@ def validate_intrinsic_if_items(items: Any) -> None:
     ValueError
         If the items are invalid
     """
-    if not isinstance(items, list) or len(items) != 3:
-        raise ValueError("Fn::If requires 3 arguments")
+    if not isinstance(items, list) or len(items) != _NUM_ARGUMENTS_REQUIRED_IN_IF:
+        raise ValueError(f"Fn::If requires {_NUM_ARGUMENTS_REQUIRED_IN_IF} arguments")
 
 
 def is_intrinsic_no_value(_input: Any) -> bool:
@@ -219,13 +219,13 @@ def get_logical_id_from_intrinsic(_input: Any) -> Optional[str]:
 
     # Fn::GetAtt: [<logical-id>, <attribute>]
     v = _input.get("Fn::GetAtt")
-    if isinstance(v, list) and len(v) == 2 and isinstance(v[0], str):
+    if isinstance(v, list) and len(v) == _NUM_ARGUMENTS_REQUIRED_IN_GETATT and isinstance(v[0], str):
         return v[0]
 
     # Fn::GetAtt: <logical-id>.<attribute>
     if isinstance(v, str):
         tokens = v.split(".")
-        if len(tokens) == 2:
+        if len(tokens) == _NUM_ARGUMENTS_REQUIRED_IN_GETATT:
             return tokens[0]
 
     return None
