@@ -1,21 +1,22 @@
-from typing import Any, Dict, Tuple
-
-import boto3
+import copy
 import json
-from botocore.config import Config
-from botocore.exceptions import ClientError, EndpointConnectionError
 import logging
 from time import sleep
-import copy
+from typing import Any, Dict, List, Optional, Tuple
 
+import boto3
+from botocore.client import BaseClient
+from botocore.config import Config
+from botocore.exceptions import ClientError, EndpointConnectionError
+
+from samtranslator.intrinsics.actions import FindInMapAction
+from samtranslator.intrinsics.resolver import IntrinsicsResolver
 from samtranslator.metrics.method_decorator import cw_timer
 from samtranslator.model.exceptions import InvalidResourceException
 from samtranslator.plugins import BasePlugin
 from samtranslator.plugins.exceptions import InvalidPluginException
 from samtranslator.public.sdk.resource import SamResourceType
 from samtranslator.public.sdk.template import SamTemplate
-from samtranslator.intrinsics.resolver import IntrinsicsResolver
-from samtranslator.intrinsics.actions import FindInMapAction
 from samtranslator.region_configuration import RegionConfiguration
 from samtranslator.utils.constants import BOTO3_CONNECT_TIMEOUT
 from samtranslator.validator.value_validator import sam_expect
@@ -50,7 +51,13 @@ class ServerlessAppPlugin(BasePlugin):
     LOCATION_KEY = "Location"
     TEMPLATE_URL_KEY = "TemplateUrl"
 
-    def __init__(self, sar_client=None, wait_for_template_active_status=False, validate_only=False, parameters=None):  # type: ignore[no-untyped-def]
+    def __init__(
+        self,
+        sar_client: Optional[BaseClient] = None,
+        wait_for_template_active_status: bool = False,
+        validate_only: bool = False,
+        parameters: Optional[Dict[str, Any]] = None,
+    ):
         """
         Initialize the plugin.
 
@@ -62,9 +69,9 @@ class ServerlessAppPlugin(BasePlugin):
         super().__init__()
         if parameters is None:
             parameters = {}
-        self._applications = {}
-        self._in_progress_templates = []
-        self._sar_client = sar_client
+        self._applications: Dict[Tuple[str, str], Any] = {}
+        self._in_progress_templates: List[Tuple[str, str]] = []
+        self.__sar_client = sar_client
         self._wait_for_template_active_status = wait_for_template_active_status
         self._validate_only = validate_only
         self._parameters = parameters
@@ -75,6 +82,15 @@ class ServerlessAppPlugin(BasePlugin):
             message = "Cannot set both validate_only and wait_for_template_active_status flags to True."
             raise InvalidPluginException(ServerlessAppPlugin.__name__, message)
 
+    @property
+    def _sar_client(self) -> BaseClient:
+        # Lazy initialization of the client- create it when it is needed
+        if not self.__sar_client:
+            # a SAR call could take a while to finish, leaving the read_timeout default (60s).
+            client_config = Config(connect_timeout=BOTO3_CONNECT_TIMEOUT)
+            self.__sar_client = boto3.client("serverlessrepo", config=client_config)
+        return self.__sar_client
+
     @staticmethod
     def _make_app_key(app_id: Any, semver: Any) -> Tuple[str, str]:
         """Generate a key that is always hashable."""
@@ -95,10 +111,9 @@ class ServerlessAppPlugin(BasePlugin):
         intrinsic_resolvers = self._get_intrinsic_resolvers(template_dict.get("Mappings", {}))  # type: ignore[no-untyped-call]
 
         service_call = None
-        if self._validate_only:
-            service_call = self._handle_get_application_request
-        else:
-            service_call = self._handle_create_cfn_template_request
+        service_call = (
+            self._handle_get_application_request if self._validate_only else self._handle_create_cfn_template_request
+        )
         for logical_id, app in template.iterate({SamResourceType.Application.value}):
             if not self._can_process_application(app):  # type: ignore[no-untyped-call]
                 # Handle these cases in the on_before_transform_resource event
@@ -128,11 +143,6 @@ class ServerlessAppPlugin(BasePlugin):
                         raise InvalidResourceException(
                             logical_id, "Serverless Application Repository is not available in this region."
                         )
-                    # Lazy initialization of the client- create it when it is needed
-                    if not self._sar_client:
-                        # a SAR call could take a while to finish, leaving the read_timeout default (60s).
-                        client_config = Config(connect_timeout=BOTO3_CONNECT_TIMEOUT)
-                        self._sar_client = boto3.client("serverlessrepo", config=client_config)
                     self._make_service_call_with_retry(service_call, app_id, semver, key, logical_id)  # type: ignore[no-untyped-call]
                 except InvalidResourceException as e:
                     # Catch all InvalidResourceExceptions, raise those in the before_resource_transform target.
@@ -422,17 +432,17 @@ class ServerlessAppPlugin(BasePlugin):
         return resource_type == self.SUPPORTED_RESOURCE_TYPE
 
     def _get_application(self, app_id, semver):  # type: ignore[no-untyped-def]
-        return self._sar_client.get_application(
+        return self._sar_client.get_application(  # type: ignore[attr-defined]
             ApplicationId=self._sanitize_sar_str_param(app_id), SemanticVersion=self._sanitize_sar_str_param(semver)  # type: ignore[no-untyped-call]
         )
 
     def _create_cfn_template(self, app_id, semver):  # type: ignore[no-untyped-def]
-        return self._sar_client.create_cloud_formation_template(
+        return self._sar_client.create_cloud_formation_template(  # type: ignore[attr-defined]
             ApplicationId=self._sanitize_sar_str_param(app_id), SemanticVersion=self._sanitize_sar_str_param(semver)  # type: ignore[no-untyped-call]
         )
 
     def _get_cfn_template(self, app_id, template_id):  # type: ignore[no-untyped-def]
-        return self._sar_client.get_cloud_formation_template(
+        return self._sar_client.get_cloud_formation_template(  # type: ignore[attr-defined]
             ApplicationId=self._sanitize_sar_str_param(app_id),  # type: ignore[no-untyped-call]
             TemplateId=self._sanitize_sar_str_param(template_id),  # type: ignore[no-untyped-call]
         )
