@@ -1,15 +1,18 @@
-import boto3
 import copy
+from typing import Any, Dict, Optional
+
+import boto3
+from boto3 import Session
 
 from samtranslator.translator.arn_generator import ArnGenerator, NoRegionFound
 
 
-class SamParameterValues(object):
+class SamParameterValues:
     """
     Class representing SAM parameter values.
     """
 
-    def __init__(self, parameter_values):  # type: ignore[no-untyped-def]
+    def __init__(self, parameter_values: Dict[Any, Any]):
         """
         Initialize the object given the parameter values as a dictionary
 
@@ -18,7 +21,7 @@ class SamParameterValues(object):
 
         self.parameter_values = copy.deepcopy(parameter_values)
 
-    def add_default_parameter_values(self, sam_template):  # type: ignore[no-untyped-def]
+    def add_default_parameter_values(self, sam_template: Dict[str, Any]) -> Any:
         """
         Method to read default values for template parameters and merge with user supplied values.
 
@@ -60,7 +63,9 @@ class SamParameterValues(object):
             if param_name not in self.parameter_values and isinstance(value, dict) and "Default" in value:
                 self.parameter_values[param_name] = value["Default"]
 
-    def add_pseudo_parameter_values(self, session=None):  # type: ignore[no-untyped-def]
+        return None
+
+    def add_pseudo_parameter_values(self, session: Optional[Session] = None) -> None:
         """
         Add pseudo parameter values
         :return: parameter values that have pseudo parameter in it
@@ -76,4 +81,4 @@ class SamParameterValues(object):
             self.parameter_values["AWS::Region"] = session.region_name
 
         if "AWS::Partition" not in self.parameter_values:
-            self.parameter_values["AWS::Partition"] = ArnGenerator.get_partition_name(session.region_name)  # type: ignore[no-untyped-call]
+            self.parameter_values["AWS::Partition"] = ArnGenerator.get_partition_name(session.region_name)
