@@ -1,17 +1,41 @@
-import boto3
-
+from functools import lru_cache
 from typing import Optional
 
+import boto3
+
 
 class NoRegionFound(Exception):
     pass
 
 
-class ArnGenerator(object):
-    BOTO_SESSION_REGION_NAME = None
+@lru_cache(maxsize=1)  # Only need to cache one as once deployed, it is not gonna deal with another region.
+def _get_region_from_session() -> str:
+    return boto3.session.Session().region_name
+
+
+@lru_cache(maxsize=1)  # Only need to cache one as once deployed, it is not gonna deal with another region.
+def _region_to_partition(region: str) -> str:
+    # setting default partition to aws, this will be overwritten by checking the region below
+    region_string = region.lower()
+    if region_string.startswith("cn-"):
+        return "aws-cn"
+    if region_string.startswith("us-iso-"):
+        return "aws-iso"
+    if region_string.startswith("us-isob"):
+        return "aws-iso-b"
+    if region_string.startswith("us-gov"):
+        return "aws-us-gov"
+
+    return "aws"
+
+
+class ArnGenerator:
+    BOTO_SESSION_REGION_NAME: Optional[str] = None
 
     @classmethod
-    def generate_arn(cls, partition, service, resource, include_account_id=True):  # type: ignore[no-untyped-def]
+    def generate_arn(
+        cls, partition: str, service: str, resource: str, include_account_id: Optional[bool] = True
+    ) -> str:
         if not service or not resource:
             raise RuntimeError("Could not construct ARN for resource.")
 
@@ -52,10 +76,11 @@ class ArnGenerator(object):
             # Use Boto3 to get the region where code is running. This uses Boto's regular region resolution
             # mechanism, starting from AWS_DEFAULT_REGION environment variable.
 
-            if ArnGenerator.BOTO_SESSION_REGION_NAME is None:
-                region = boto3.session.Session().region_name
-            else:
-                region = ArnGenerator.BOTO_SESSION_REGION_NAME  # type: ignore[unreachable]
+            region = (
+                _get_region_from_session()
+                if ArnGenerator.BOTO_SESSION_REGION_NAME is None
+                else ArnGenerator.BOTO_SESSION_REGION_NAME
+            )
 
         # If region is still None, then we could not find the region. This will only happen
         # in the local context. When this is deployed, we will be able to find the region like
@@ -63,17 +88,4 @@ class ArnGenerator(object):
         if region is None:
             raise NoRegionFound("AWS Region cannot be found")
 
-        # setting default partition to aws, this will be overwritten by checking the region below
-        partition = "aws"
-
-        region_string = region.lower()
-        if region_string.startswith("cn-"):
-            partition = "aws-cn"
-        elif region_string.startswith("us-iso-"):
-            partition = "aws-iso"
-        elif region_string.startswith("us-isob"):
-            partition = "aws-iso-b"
-        elif region_string.startswith("us-gov"):
-            partition = "aws-us-gov"
-
-        return partition
+        return _region_to_partition(region)
